// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

/*
** BER Tags
**
** 000x xxxx - Universal, primitive                     0x0n, 0x1n
** 001x xxxx - Universal, constructed                   0x2n, 0x3n
** 010x xxxx - Application, primitive                   0x4n, 0x5n
** 011x xxxx - Application, constructed                 0x6n, 0x7n
** 100x xxxx - Context-specific, primitive              0x8n, 0x9n
** 101x xxxx - Context-specific, constructed            0xAn, 0xBn
** 110x xxxx - Private, primitive                       0xCn, 0xDn
** 111x xxxx - Private, constructed                     0xEn, 0xFn
**
*/

#include <windows.h>
#include "BERtlv.h"

/*
** Find a BER-TLV in a template
*/
int tlvFindBERTag(DWORD templatetag, DWORD tag, BYTE *tlv, BYTE *taglen, DWORD *len, BYTE *lenlen)
{
    BYTE ttaglen, tlenlen;
    DWORD ttag;
    int offset;

    ttag = getTagLength(tlv, &ttaglen, len, &tlenlen);

    if((ttag != 0) && (ttag == templatetag) && ((offset = tlvFindBERNext(tag, tlv + ttaglen + tlenlen, *len, taglen, len, lenlen)) != -1))
    {
        return ttaglen + tlenlen + offset;
    }
    else
    {
        return -1;
    }
}

int tlvFindBERNext(DWORD tag, BYTE *tlv, DWORD length, BYTE *taglen, DWORD *len, BYTE *lenlen)
{
    BYTE *p, *q;
    DWORD ttag = 0;

    p = tlv;
    q = p + length;

    while(p < q)
    {
        if((ttag = getTagLength(p, taglen, len, lenlen)) == 0)
        {
            return -1;
        } 
        else if(ttag == tag)
        {
            break;
        }
        else
        {
            p += *taglen + *lenlen + *len;
        }
    }

    return (p < q) ? (int)(p-tlv) : -1;
}

DWORD getTagLength(BYTE *tlv, BYTE *taglen, DWORD *len, BYTE *lenlen)
{
    int i;
    DWORD tag = (DWORD)tlv[0];

    if(tag == 0)
    {
        *taglen = 0;
        *len = 0;
        *lenlen = 0;
        return 0;
    } 
    else if((tlv[0] & 0x1F) == 0x1F)
    {
        tlv++;
        if(tlv[0] & 0x80)
        {
            tag <<= 16;
            tag |= (DWORD)(((DWORD)(tlv[0]<<8)) | ((DWORD)(tlv[1]&0x7F)));
            *taglen = 3;
            tlv += 2;
        }
        else
        {
            tag <<= 8;
            tag |= (DWORD)tlv[0];
            *taglen = 2;
            tlv++;
        }
    }
    else
    {
        *taglen = 1;
        tlv++;
    }

    if(tlv[0] & 0x80)
    {
        *lenlen = (*tlv++ & 0x07);
        for(*len = tlv[0], i = 1; i < *lenlen; i++)
        {
            *len = (DWORD)(((DWORD)(*len << 8)) | ((DWORD)tlv[i]));
        }
        (*lenlen)++;
    }
    else
    {
        *lenlen = 1;
        *len = (DWORD)tlv[0];
    }

    return tag;
}

DWORD tlvIntegerValue(BYTE *tlv, DWORD len)
{
   DWORD value = 0;

   switch (len) 
   {
      case 4: value |= *tlv++; value <<= 8;
      case 3: value |= *tlv++; value <<= 8;
      case 2: value |= *tlv++; value <<= 8;
      case 1: value |= *tlv++;
   }

   return value;
}

BYTE getTlvValueOffset(BYTE* tlv)
{
	BYTE valueOffset = 2;

	if (tlv[1] & 0x80)
		valueOffset += (tlv[1] & 0x7F);

	return valueOffset;
}
