/* file base64util.c
 *
 * NIST PIV Demo CSP -- Cryptographic Service Provider
 *
 * Designed and Developed by Electrosoft (www.electrosoft-inc.com) based on requirements
 * from the National Institute of Standards and Technology (NIST).
 *
 * This file codes and documents some debug functions.
 */

#include <stdlib.h>
#include <stdio.h>

#ifdef WIN32
#include <windows.h>
#endif
#include "csp-debug.h"

#define ATL_BASE64_FLAG_NONE    0
#define ATL_BASE64_FLAG_NOPAD   1
#define ATL_BASE64_FLAG_NOCRLF  2

int Base64EncodeGetRequiredLength(int nSrcLen, DWORD dwFlags/*=ATL_BASE64_FLAG_NONE*/)
{
		int nCRLFs;
		int nOnLastLine;
        int nRet = nSrcLen*4/3;


        if ((dwFlags & ATL_BASE64_FLAG_NOPAD) == 0)
                nRet += nSrcLen % 3;


        nCRLFs = nRet / 76 + 1;
        nOnLastLine = nRet % 76;


        if (nOnLastLine)
        {
                if (nOnLastLine % 4)
                        nRet += 4-(nOnLastLine % 4);
        }


        nCRLFs *= 2;


        if ((dwFlags & ATL_BASE64_FLAG_NOCRLF) == 0)
                nRet += nCRLFs;


        return nRet;
}

int Base64Encode(const BYTE *pbSrcData,
        int nSrcLen,
        LPSTR szDest,
        int *pnDestLen,
        DWORD dwFlags)
{

		int nWritten;
        int nLen1;
        int nLen2;
        int nLen3;
		int i,j, n, k;
		DWORD dwCurr;
		BYTE b;

static const char s_chBase64EncodingTable[64] = {
                'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
'O', 'P', 'Q',
                'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e',
'f', 'g',       'h',
                'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
'w', 'x', 'y',
                'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/' };

        if (!pbSrcData || !szDest || !pnDestLen)
        {
			DEBUG(3, " ######### BASE64 FAILING (1) ######\n");
            return FALSE;
        }


        if(*pnDestLen < Base64EncodeGetRequiredLength(nSrcLen, dwFlags))
        {
            //ATLASSERT(FALSE);
			DEBUG(3, " ######### BASE64 FAILING (2) ######\n");
            return FALSE;
        }


        nWritten = 0;
        nLen1 = (nSrcLen/3)*4;
        nLen2 = ( nLen1/76 );
        nLen3 = ( 19 );


        for (i=0; i<=nLen2; i++)
        {
                if (i==nLen2)
                        nLen3 = (nLen1%76)/4;


                for (j=0; j<nLen3; j++)
                {
                        dwCurr = (0);
                        for (n=0; n<3; n++)
                        {
                                dwCurr |= *pbSrcData++;
                                dwCurr <<= 8;
                        }
                        for (k=0; k<4; k++)
                        {
                                b = (BYTE)(dwCurr>>26);
                                *szDest++ = s_chBase64EncodingTable[b];
                                dwCurr <<= 6;
                        }
                }
                nWritten+= nLen3*4;


                if ((dwFlags & ATL_BASE64_FLAG_NOCRLF)==0)
                {
                        *szDest++ = '\r';
                        *szDest++ = '\n';
                        nWritten+= 2;
                }
        }


        if (nWritten && (dwFlags & ATL_BASE64_FLAG_NOCRLF)==0)
        {
                szDest-= 2;
                nWritten -= 2;
        }


        nLen2 = nSrcLen%3 ? nSrcLen%3 + 1 : 0;
        if (nLen2)
        {
                dwCurr = (0);
                for (n=0; n<3; n++)
                {
                        if (n<(nSrcLen%3))
                                dwCurr |= *pbSrcData++;
                        dwCurr <<= 8;
                }
                for (k=0; k<nLen2; k++)
                {
                        b = (BYTE)(dwCurr>>26);
                        *szDest++ = s_chBase64EncodingTable[b];
                        dwCurr <<= 6;
                }
                nWritten+= nLen2;
                if ((dwFlags & ATL_BASE64_FLAG_NOPAD)==0)
                {
                        nLen3 = nLen2 ? 4-nLen2 : 0;
                        for (j=0; j<nLen3; j++)
                        {
                                *szDest++ = '=';
                        }
                        nWritten+= nLen3;
                }
        }


        *pnDestLen = nWritten;
        return TRUE;
}
